﻿//////////////////////////////////////////////
// MainSystem.h
//
//////////////////////////////////////////////

/// Defines / Macros -------------------------

#pragma once

/// Forward decl -----------------------------

namespace nkGraphics
{
	class GraphicSystem ;
	class InputSystem ;
	class RenderContext ;
}

/// Includes ---------------------------------

// nkGraphics
#include "../Dll/DllDefines.h"

// Common
#include <nilkinsCommon/Patterns/SingletonClass.h>

/// Class ------------------------------------

namespace nkGraphics
{
	class DLL_GRAPHICS_EXPORT MainSystem final : public nkCommon::SingletonClass<nkGraphics::MainSystem>
	{
		friend class nkCommon::SingletonClass<nkGraphics::MainSystem> ;

		private :
		
			// Attributs
			// Nos systèmes
			GraphicSystem* _graphicSys ;
			InputSystem* _inputSys ;

			// Témoins passés
			bool _initialized ;
			bool _hasRunToContinue ;

		private :

			// Functions
			// Constructor
			MainSystem () ;
		
		public :
		
			// Destructor
			~MainSystem () ;

			// Getters
			GraphicSystem* getGraphicSystem () const ;
			InputSystem* getInputSystem () const ;
			bool isInitialized () const ;
			bool getHasRunToContinue () const ;

			// Setters
			void setHasRunToContinue (bool value) ;

			// To start
			bool autoConfigure () ;
			bool initialize () ;
			void run (RenderContext* context) ;

			// To end
			void shutdown () ;

			// Step by step
			bool frame (RenderContext* context) ;
			void tickForNewFrame (RenderContext* context) ;
			void tickInputSystem () ;
	} ;
}